"use strict";

exports.__esModule = true;
exports.default = void 0;

var _rax = require("rax");

var _raxView = _interopRequireDefault(require("rax-view"));

var _universalTransition = _interopRequireDefault(require("universal-transition"));

var _universalEnv = require("universal-env");

var _index = _interopRequireDefault(require("./index.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var _styleSheet = _index.default;
var bodyEl, originalBodyOverflow;
var modalCount = 0;

if (_universalEnv.isWeb) {
  bodyEl = document.body;
}

function stopPropagation(event) {
  if (_universalEnv.isWeb) {
    event.stopPropagation();
  }
}

function stopEventEffect(event) {
  if (_universalEnv.isWeb) {
    event.preventDefault();
    event.stopPropagation();
  }
}

function Modal(props) {
  var visible = props.visible,
      onMaskClick = props.onMaskClick,
      _props$maskCanBeClick = props.maskCanBeClick,
      maskCanBeClick = _props$maskCanBeClick === void 0 ? true : _props$maskCanBeClick,
      _props$maskStyle = props.maskStyle,
      maskStyle = _props$maskStyle === void 0 ? {} : _props$maskStyle,
      _props$contentStyle = props.contentStyle,
      contentStyle = _props$contentStyle === void 0 ? {} : _props$contentStyle,
      onShow = props.onShow,
      onHide = props.onHide,
      children = props.children,
      _props$animation = props.animation,
      animation = _props$animation === void 0 ? true : _props$animation,
      _props$delay = props.delay,
      delay = _props$delay === void 0 ? 0 : _props$delay;
  var _props$duration = props.duration,
      duration = _props$duration === void 0 ? [300, 300] : _props$duration;

  if (typeof duration === 'number') {
    duration = [duration, duration];
  } else if (duration.length === 1) {
    // If duration's length is one, in and out are the same.
    duration = [duration[0], duration[0]];
  }

  var maskRef = (0, _rax.useRef)(null);

  var _useState = (0, _rax.useState)(false),
      visibleState = _useState[0],
      setVisibleState = _useState[1];

  var _useState2 = (0, _rax.useState)(null),
      height = _useState2[0],
      setHeight = _useState2[1];

  if (_universalEnv.isWeex) {
    var dom = __weex_require__('@weex-module/dom');

    dom.getComponentRect('viewport', function (e) {
      setHeight(e.size.height);
    });
  } else if (_universalEnv.isWeb) {
    setHeight('100vh');
  }

  var animate = function (show, callback) {
    maskRef.__animationValid = true;
    var animateDuration = show ? duration[0] : duration[1]; // Record animation execute timer

    maskRef.__timer = setTimeout(function () {
      maskRef.__animationValid = false;

      if (show && maskRef.current) {
        // When target state is show, it need set modal opacity to 1
        maskRef.current.style.opacity = '1';
      }

      callback && callback();
    }, animateDuration);
    (0, _universalTransition.default)(maskRef.current, {
      opacity: show ? 1 : 0
    }, {
      timingFunction: 'ease',
      delay: delay,
      duration: animateDuration
    }, function () {
      // Ensure animation timer hasn't been executed
      if (maskRef.__animationValid) {
        clearTimeout(maskRef.__timer);

        if (show && maskRef.current) {
          // When target state is show, it need set modal opacity to 1
          maskRef.current.style.opacity = '1';
        }

        callback && callback();
      }
    });
  };

  var show = function () {
    if (!maskRef.__pendingShow) {
      maskRef.__pendingShow = true; // Stop pending hide action

      if (maskRef.__pendingHide) {
        maskRef.__pendingHide = false;
      } else {
        modalCount++;
      }

      if (_universalEnv.isWeb) {
        // Only when current modal count is 1, it need record origin body overflow
        if (modalCount === 1) {
          originalBodyOverflow = bodyEl.style.overflow;
        }

        bodyEl.style.overflow = 'hidden';
      }

      setVisibleState(true);

      if (animation) {
        if (maskRef.current) {
          maskRef.current.style.opacity = '0';
        }

        animate(true, function () {
          onShow && onShow();
        });
      } else {
        if (maskRef.current) {
          maskRef.current.style.opacity = '1';
        }

        onShow && onShow();
      }
    }
  };

  var hideAction = function () {
    if (_universalEnv.isWeb && modalCount === 1) {
      bodyEl.style.overflow = originalBodyOverflow;
    }

    modalCount--;
    setVisibleState(false);
    onHide && onHide();
  };

  var hide = function (withAnimate) {
    if (withAnimate === void 0) {
      withAnimate = animation;
    }

    if (visibleState && !maskRef.__pendingHide) {
      maskRef.__pendingHide = true;
      maskRef.__pendingShow = false;

      if (withAnimate) {
        // execute hide animation on element that is already hidden will cause bug
        animate(false, function () {
          // Only when pending hide execute hide action
          if (maskRef.__pendingHide) {
            hideAction();
          }
        });
      } else {
        hideAction();
      }
    }
  };

  (0, _rax.useEffect)(function () {
    // When a new modal mounted, modal count ++
    return function () {
      // Clear timer
      clearTimeout(maskRef.__timer);
      hide(false);
    };
  }, []);
  (0, _rax.useEffect)(function () {
    // if state is unequal to props trigger show or hide
    if (visible !== visibleState) {
      visible ? show() : hide();
    } else if (visible) {
      // When visible changed to true, while visibleState didn't change, it should trigger show
      show();
    }
  }, [visible]);
  (0, _rax.useEffect)(function () {
    // Record mask show state
    maskRef.__pendingShow = false; // Record mask hide state

    maskRef.__pendingHide = false;
  }, [visibleState]);
  return (0, _rax.createElement)(_raxView.default, {
    className: "rax-modal-mask",
    style: Object.assign({}, _styleSheet["rax-modal-mask"], _extends({}, maskStyle, {
      visibility: visibleState ? 'visible' : 'hidden',
      height: height || 0
    })),
    onTouchMove: stopEventEffect,
    onClick: function onClick() {
      if (props.hasOwnProperty('onMaskClick')) {
        onMaskClick && onMaskClick();
      } else if (maskCanBeClick) {
        // When maskCanBeClick is true, should trigger hide
        hide();
      }
    },
    ref: maskRef
  }, (0, _rax.createElement)(_raxView.default, {
    className: "rax-modal-main",
    style: Object.assign({}, _styleSheet["rax-modal-main"], contentStyle),
    onClick: stopPropagation,
    onTouchMove: stopPropagation
  }, children));
}

Modal.displayName = 'Modal';
var _default = Modal;
exports.default = _default;