"use strict";

var noop = function () {};

var triggleVisible = function (instance, props) {
  var visible = props.visible,
      onShow = props.onShow,
      onHide = props.onHide;
  instance.setData({
    visibility: visible
  }, function () {
    if (visible) {
      onShow && onShow();
    } else {
      onHide && onHide();
    }
  });
};

var updateData = function (instance, props) {
  if (props.delay) {
    setTimeout(function () {
      triggleVisible(instance, props);
    }, props.delay);
  } else {
    triggleVisible(instance, props);
  }

  var maskStyleState = computeStyle(props.maskStyle);
  var contentStyleState = computeStyle(props.contentStyle);
  instance.setData({
    maskStyleState: maskStyleState,
    contentStyleState: contentStyleState
  });
};

Component({
  data: {
    visibility: false,
    maskStyleState: '',
    contentStyleState: ''
  },
  props: {
    visible: false,
    maskStyle: {},
    contentStyle: {},
    onShow: noop,
    onHide: noop,
    maskCanBeClick: true,
    delay: 0,
    duration: 300,
    onMaskClick: noop
  },
  didUpdate: function didUpdate() {
    // eslint-disable-next-line no-undef
    if (!my.canIUse('component2')) {
      updateData(this, this.props);
    }
  },
  deriveDataFromProps: function deriveDataFromProps(nextProps) {
    updateData(this, nextProps);
  },
  methods: {
    onClick: function onClick() {
      var _this$props = this.props,
          maskCanBeClick = _this$props.maskCanBeClick,
          onHide = _this$props.onHide,
          onMaskClick = _this$props.onMaskClick;

      if (typeof onMaskClick === 'function' && onMaskClick !== noop) {
        onMaskClick();
      } else if (maskCanBeClick) {
        onHide && onHide();
      }
    }
  }
});

function computeStyle(style) {
  var handledStyle = '';
  Object.keys(style).forEach(function (key) {
    handledStyle = handledStyle + (key + ": " + style[key] + ";");
  });
  return handledStyle;
}